@@ -181,6 +181,8 @@ application_lookup_or_create (const char *app_id, gboolean user_owned)
 
 	g_return_val_if_fail(app_id != NULL, NULL);
 
+	gnome_vfs_application_registry_init ();
+
 	application = g_hash_table_lookup (global_applications, app_id);
 	if (application != NULL) {
 		if ( ! user_owned) {
@@ -537,6 +539,7 @@ application_sync (Application *application, FILE *fp)
 	GList *li;
 
 	g_return_if_fail (application != NULL);
+	g_return_if_fail (application->app_id != NULL);
 	g_return_if_fail (fp != NULL);
 
 	fprintf (fp, "%s\n", application->app_id);
@@ -610,7 +613,7 @@ application_add_key (Application *application, const char *key,
 			next_value = strtok (NULL, ", \t");
 		}
 		g_free(value_copy);
-		return;		   
+		/* fall through so that we can store the values as keys too */
 	}
 	else if (strcmp (key, "expects_uris") == 0) {
 		if (strcmp (value, "non-file") == 0) {
@@ -968,12 +971,12 @@ gnome_vfs_application_registry_init (void)
 			  user_registry_dir.dirname);
 	}
 
-	/*
-	 * Load
+	/* Things have been initialized flag it as ready so that we can load
+	 * the applications without attempting to reinitialize
 	 */
-	load_application_info ();
-
 	gnome_vfs_application_registry_initialized = TRUE;
+
+	load_application_info ();
 }
 
 /*
