@@ -37,6 +37,7 @@
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <locale.h>
 
 #include <libgnomevfs/gnome-vfs-mime.h>
 
@@ -47,7 +48,7 @@
 #include <libgnomevfs/gnome-vfs-module-shared.h>
 
 #define LINESIZE 1024
-#define CDE_ICON_NAME_CACHE "/tmp/g-cdemenu.tmp"
+#define CDE_ICON_NAME_CACHE "/.gnome/g-cdeiconnames"
 
 static GSList *cdemenufiles;
 
@@ -105,8 +106,13 @@ expand_env_vars(char *s)
 				tmp2 = tmp+(strlen(tmp))-1; *tmp2='\0';
 			}
 			tmp3 = g_getenv(tmp);
+			if (!tmp3 && strstr(tmp, "LC_CTYPE"))
+				/* It's not always the case $LC_CTYPE or $LANG is set */
+				tmp3 = (char*)setlocale(LC_CTYPE, NULL);
 			g_free(*token);
-			*token = g_strdup(tmp3);
+			/* don't do g_strdup (s?s1:s2) as that doesn't work with
+			   certain gcc 2.96 versions */
+			*token = tmp3 ? g_strdup(tmp3) : g_strdup("");
 		}
 	}
 	expanded = g_strjoinv("/", tokens);
@@ -148,16 +154,20 @@ get_icon_for_action(char *action)
 	FILE *file;
 	char **dir, *expandeddir, *fullpath=NULL;
 	char *tmp=NULL;
+	char *icon_name_cache_path;
 
-	char *iconsearchpath[] = {"/usr/dt/appconfig/icons/$LANG",
-			"/etc/dt/appconfig/icons/$LANG",
-   			"$HOME/.dt/icons", NULL };
+	char *iconsearchpath[] = {"/usr/dt/appconfig/icons/$LC_CTYPE",
+                       "/etc/dt/appconfig/icons/$LC_CTYPE",
+                       "$HOME/.dt/icons",
+                       "/usr/dt/appconfig/icons/C", NULL};
 
 	/*if action has arg strip the arg*/
 	if ((tmp=strchr(action,'\"'))) {*tmp='\0'; tmp=NULL;}
 	
 	/*Return NULL icon if icon DB does not exist*/
-	file = fopen(CDE_ICON_NAME_CACHE,"r");
+	icon_name_cache_path = g_strconcat (g_get_home_dir (), CDE_ICON_NAME_CACHE, NULL);
+	file = fopen(icon_name_cache_path,"r");
+	g_free(icon_name_cache_path);
 	if (!file) return NULL;
 
 	while (fgets(line,LINESIZE,file) != NULL){
@@ -231,12 +241,35 @@ do_open (GnomeVFSMethod *method,
 			if (line[0] == '#') continue;
 			else if (line[0] == '}') break;
 			else if (strstr(line,"f.title")) {
+				char *utf8_name = NULL;
+				
 				tmp = strchr(line,'\"'); tmp ++;
 				tmp2 = strchr(tmp,'\"'); if(tmp2) *tmp2='\0';
-				dtfile->contents= g_strdup_printf("[Desktop "
-					"Entry]\nName=%s\nComment=%s\nIcon=%s\nType=Directory\n",
-					 tmp,tmp,get_icon_for_menu(tmp));
+
+				utf8_name = g_locale_to_utf8 (tmp, -1,
+							      NULL, NULL,
+							      NULL);
+				/* fallback to avoid crash */
+				if (utf8_name == NULL)
+					utf8_name = g_strdup (tmp);
+
+				dtfile->contents = g_strdup_printf 
+					("[Desktop Entry]\n"
+					 "Encoding=UTF-8\n"
+					 /* Note that we include the utf-8
+					  * translated name without a language
+					  * specifier, but that's OK, it will
+					  * work */
+					 "Name=%s\n"
+					 "Comment=%s\n"
+					 "Icon=%s\n"
+					 "Type=Directory\n",
+					 utf8_name, 
+					 utf8_name,
+					 get_icon_for_menu(tmp));
 				/* Title is found, dont look for secondary title*/
+
+				g_free (utf8_name);
 				break;
 			}
 		}
@@ -246,28 +279,75 @@ do_open (GnomeVFSMethod *method,
 			else if (line[0] == '}') break;
 			else if (strstr(line,name)){
 				if((tmp = strstr(line,"f.action")) != NULL){
+					char *utf8_name = NULL;
+
 					tmp+=8;
 					end = strchr(tmp,'\n');
 					if (end) *end='\0';
 					exec=g_strdup_printf("dtaction %s",tmp);
 					icon=get_icon_for_action(tmp);
 					if (!icon) icon=g_strdup("NONE");
-					dtfile->contents= g_strdup_printf("[Desktop Entry]\nName=%s"
-								"\nComment=%s\nExec=%s\nIcon=%s\nTerminal=0\n"
-								"Type=Application\n", name, name, exec, icon);
+
+					utf8_name = g_locale_to_utf8 (name, -1,
+								      NULL,
+								      NULL,
+								      NULL);
+					/* fallback to avoid crash */
+					if (utf8_name == NULL)
+						utf8_name = g_strdup (name);
+
+					dtfile->contents= g_strdup_printf
+						("[Desktop Entry]\n"
+						 "Encoding=UTF-8\n"
+						 /* Note that we include the
+						  * utf-8 translated name
+						  * without a language
+						  * specifier, but that's
+						  * OK, it will work */
+						 "Name=%s\n"
+						 "Comment=%s\n"
+						 "Exec=%s\n"
+						 "Icon=%s\n"
+						 "Terminal=0\n"
+						 "Type=Application\n",
+						 utf8_name, utf8_name,
+						 exec, icon);
+					g_free (utf8_name);
 					g_free(icon);
 				} else if ((tmp = strstr(line,"f.exec"))!=NULL){
+					char *utf8_name = NULL;
+
 					tmp+=6;
 					tmp = g_strstrip(tmp);
 					/* strip off quotes */
 					if (*tmp == '\"') tmp++;
 					end = tmp+(strlen(tmp))-1;
 					if (*end == '\"') *end = '\0';
-					dtfile->contents= g_strdup_printf("[Desktop Entry]\nName=%s"
-								"\nComment=%s\nExec=%s\n"
-								"Icon=\n"
-								"Terminal=0\n"
-			  		  			"Type=Application\n", name, name, tmp);
+
+					utf8_name = g_locale_to_utf8 (name, -1,
+								      NULL,
+								      NULL,
+								      NULL);
+					/* fallback to avoid crash */
+					if (utf8_name == NULL)
+						utf8_name = g_strdup (name);
+
+					dtfile->contents= g_strdup_printf
+						("[Desktop Entry]\n"
+						 "Encoding=UTF-8\n"
+						 /* Note that we include the
+						  * utf-8 translated name
+						  * without a language
+						  * specifier, but that's
+						  * OK, it will work */
+						 "Name=%s\n"
+						 "Comment=%s\n"
+						 "Exec=%s\n"
+						 "Icon=\n"
+						 "Terminal=0\n"
+						 "Type=Application\n",
+						 utf8_name, utf8_name, tmp);
+					g_free (utf8_name);
 				}
 			}
 		}
@@ -365,7 +445,7 @@ do_read_directory (GnomeVFSMethod *method,
 					|| strstr(line,"UpdateWorkspaceMenu"))) {
 					file_info->name = g_strdup("bogus.desktop");
 					file_info->type = GNOME_VFS_FILE_TYPE_REGULAR;
- 					break;
+ 					continue;
 			}
 			else if (strstr(line,"f.action") || strstr(line,"f.exec")) {
 					tmp = strchr(line,'\"'); tmp ++;
@@ -471,9 +551,9 @@ find_all_included_files(gchar *filename)
 			while (fgets(line,LINESIZE,file) !=NULL){
 				if (line[0] == '#') continue;
 				if (line[0] == '{') continue;
-				if (line[0] == ' ') continue;
 				else if (line[0] == '}') break;
 				tmp = g_strstrip(line);
+				if (tmp[0] == '\0') continue;
 				expanded = expand_env_vars(tmp);
 				if (g_file_test(expanded,G_FILE_TEST_EXISTS)){
                		cdemenufiles = g_slist_append(cdemenufiles, 
@@ -492,13 +572,19 @@ create_cde_icon_name_cache (void)
 {
 	/* fork the dttypes command to create the icon name cache */
 	int fd, t;
+	mode_t old_mask;
+	char *icon_name_cache_path;
 	
 	if ((t = fork ()) < 0) g_error ("Unable to fork.");
 	else if (t) wait(NULL);
 	else {
-		fd = open (CDE_ICON_NAME_CACHE, O_WRONLY | O_CREAT | O_TRUNC, 0777);
+		icon_name_cache_path = g_strconcat (g_get_home_dir (), CDE_ICON_NAME_CACHE, NULL);
+		old_mask = umask(033);
+		fd = open (icon_name_cache_path, O_WRONLY | O_CREAT | O_TRUNC, 0644);
 		dup2 (fd, 1);
 		fchmod (fd, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH); 
+		g_free(icon_name_cache_path);
+		umask(old_mask);
 		execl("/usr/dt/bin/dttypes", "dttypes", "-w", "fld_name", "ICON",
 			  "-l", "fld_name", "ICON", NULL);
 	}
@@ -508,8 +594,10 @@ GnomeVFSMethod *
 vfs_module_init (const char *method_name, 
 		 const char *args)
 {
-	gchar *files[6];
+	gchar *files[7];
 	int i;
+	char *icon_name_cache_path;
+
 	parent_method = gnome_vfs_method_get ("file");
 
 	if (parent_method == NULL) {
@@ -519,21 +607,22 @@ vfs_module_init (const char *method_name,
 
 	/* find the right cde menu file to start with */
 	cdemenufiles = NULL;
-	files[0] = expand_env_vars("$HOME/.dt/$LANG/dtwmrc");
+	files[0] = expand_env_vars("$HOME/.dt/$LC_CTYPE/dtwmrc");
 	files[1] = expand_env_vars("$HOME/.dt/dtwmrc");
-	files[2] = expand_env_vars("/etc/dt/config/$LANG/sys.dtwmrc");
+	files[2] = expand_env_vars("/etc/dt/config/$LC_CTYPE/sys.dtwmrc");
 	files[3] = g_strdup("/etc/dt/config/sys.dtwmrc");
-	files[4] = expand_env_vars("/usr/dt/config/$LANG/sys.dtwmrc");
+	files[4] = expand_env_vars("/usr/dt/config/$LC_CTYPE/sys.dtwmrc");
 	files[5] = g_strdup("/usr/dt/config/sys.dtwmrc");
+	files[6] = g_strdup("/usr/dt/config/C/sys.dtwmrc");
 
-	for (i=0;i<6;i++){
+	for (i=0;i<7;i++){
 		if (g_file_test(files[i],G_FILE_TEST_EXISTS)){
 			cdemenufiles = g_slist_append(cdemenufiles,
 				g_strdup(files[i]));
 			break;
 		}	
 	}
-	for (i=0;i<6;i++) g_free(files[i]);
+	for (i=0;i<7;i++) g_free(files[i]);
 
 	/* if didnt find any menu start file then we are fecked */
 	if (cdemenufiles == NULL) return NULL;
@@ -542,9 +631,11 @@ vfs_module_init (const char *method_name,
 	find_all_included_files(cdemenufiles->data);
 
 	/*create the icon name cache if it does not exist*/
-	if (!g_file_test(CDE_ICON_NAME_CACHE, G_FILE_TEST_EXISTS))
+	icon_name_cache_path = g_strconcat (g_get_home_dir (), CDE_ICON_NAME_CACHE, NULL);
+	if (!g_file_test(icon_name_cache_path, G_FILE_TEST_EXISTS))
 		create_cde_icon_name_cache();
 
+	g_free(icon_name_cache_path);
 	return &method;
 }
 
