@@ -21,24 +21,19 @@
 
    Author: Ettore Perazzoli <ettore@comm2000.it> */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
-#ifdef HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
-
-#include <zlib.h>
-#include <time.h>
+#include <glib/galloca.h>
+#include <glib/gmessages.h>
+#include <glib/gstrfuncs.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomevfs/gnome-vfs-module.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
+#include <zlib.h>
 
-#include "gnome-vfs-module.h"
-#include "gnome-vfs-mime.h"
-#include "gnome-vfs-ops.h"
-
-
 struct _GZipMethodHandle {
 	GnomeVFSURI *uri;
 	GnomeVFSHandle *parent_handle;
@@ -279,7 +274,7 @@ skip (GnomeVFSHandle *handle,
 	guchar *tmp;
 	GnomeVFSFileSize bytes_read;
 
-	tmp = alloca (num_bytes);
+	tmp = g_alloca (num_bytes);
 
 	result = gnome_vfs_read (handle, tmp, num_bytes, &bytes_read);
 	RETURN_IF_FAIL (result);
@@ -737,7 +732,7 @@ do_get_file_info  (GnomeVFSMethod *method,
 			file_info->name[namelen-3] = '\0';
 
 		/* we can't tell the size without uncompressing it */
-		/* file_info->valid_fields &= ~GNOME_VFS_FILE_INFO_FIELDS_SIZE; */
+		//file_info->valid_fields &= ~GNOME_VFS_FILE_INFO_FIELDS_SIZE;
 
 		/* guess the mime type of the file inside */
 		/* FIXME bugzilla.eazel.com 2791: guess mime based on contents */
